@@ -10,12 +10,12 @@ def top_p(logits, thres = 0.9):
     sorted_logits, sorted_indices = torch.sort(logits, descending=True)
     cum_probs = torch.cumsum(F.softmax(sorted_logits, dim=-1), dim=-1)
 
-    sorted_indices_to_remove = cum_probs > thres
+    sorted_indices_to_remove = cum_probs > (1 - thres)
     sorted_indices_to_remove[:, 1:] = sorted_indices_to_remove[:, :-1].clone()
     sorted_indices_to_remove[:, 0] = 0
 
     sorted_logits[sorted_indices_to_remove] = float('-inf')
-    return sorted_logits.gather(1, sorted_indices)
+    return sorted_logits.scatter(1, sorted_indices, sorted_logits)
 
 def top_k(logits, thres = 0.9):
     k = int((1 - thres) * logits.shape[-1])
@@ -46,14 +46,23 @@ class TrainingWrapper(nn.Module):
 
         self.net.eval()
         out = start_tokens
+        input_mask = kwargs.pop('input_mask', None)
+
+        if input_mask is None:
+            input_mask = torch.full_like(out, True, dtype=torch.bool, device=out.device)
 
         for _ in range(seq_len):
             x = out[:, -self.max_seq_len:]
-            logits = self.net(x, **kwargs)[:, -1, :]
+            input_mask = input_mask[:, -self.max_seq_len:]
+
+            logits = self.net(x, input_mask=input_mask, **kwargs)[:, -1, :]
             filtered_logits = filter_logits_fn(logits, thres = filter_thres)
             probs = F.softmax(filtered_logits / temperature, dim=-1)
             sample = torch.multinomial(probs, 1)
+
             out = torch.cat((out, sample), dim=-1)
+            input_mask = F.pad(input_mask, (0, 1), value=True)
+
             if eos_token is not None and (sample == eos_token).all():
                 break
 
